@@ -2,7 +2,8 @@
    GNU Mailutils nntp functions.  This application interactively allows users
    to contact a nntp server.
 
-   Copyright (C) 2003, 2004, 2005, 2007, 2009 Free Software Foundation
+   Copyright (C) 2003-2005, 2007, 2009-2012, 2014-2016 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,9 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>  
@@ -42,7 +41,7 @@
 #include <mailutils/iterator.h>
 #include <mailutils/error.h>
 #include <mailutils/errno.h>
-#include <mailutils/mutil.h>
+#include <mailutils/util.h>
 #include <mailutils/cctype.h>
 #include <mailutils/cstr.h>
 
@@ -344,9 +343,8 @@ com_verbose (char *arg)
     {
       if (verbose)
 	{
-	  mu_debug_t debug;
-	  mu_debug_create (&debug, NULL);
-	  mu_debug_set_level (debug, MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
+	  mu_debug_set_category_level (MU_DEBCAT_REMOTE,
+	                               MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
 	  status = mu_nntp_set_debug (nntp, debug);
 	}
       else
@@ -394,9 +392,10 @@ com_head (char *arg)
     {
       size_t n = 0;
       char buf[128];
-      while ((mu_stream_readline (stream, buf, sizeof buf, 0, &n) == 0) && n)
+      /* FIXME: mu_stream_seek (stream, 0, MU_SEEK_SET); ? */
+      while ((mu_stream_readline (stream, buf, sizeof buf, &n) == 0) && n)
         printf ("%s", buf);
-      mu_stream_destroy (&stream, NULL);
+      mu_stream_destroy (&stream);
     }
    return status;
 }
@@ -421,11 +420,10 @@ com_body (char *arg)
 
    if (status == 0 && stream != NULL)
     {
-      size_t n = 0;
       char buf[128];
-      while ((mu_stream_readline (stream, buf, sizeof buf, 0, &n) == 0) && n)
+      while (mu_stream_readline (stream, buf, sizeof buf, 0) == 0)
         printf ("%s", buf);
-      mu_stream_destroy (&stream, NULL);
+      mu_stream_destroy (&stream);
     }
    return status;
 }
@@ -1003,8 +1001,7 @@ com_connect (char *arg)
       if (verbose)
 	com_verbose ("on");
       status =
-	mu_tcp_stream_create (&tcp, host, port,
-			   MU_STREAM_READ | MU_STREAM_NO_CHECK);
+	mu_tcp_stream_create (&tcp, host, port, MU_STREAM_READ);
       if (status == 0)
 	{
 	  mu_nntp_set_carrier (nntp, tcp);
