@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2003, 2005, 2006,
-   2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2003, 2005-2012, 2014-2016 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -24,15 +22,16 @@ static int select_flags;
 /* select          ::= "SELECT" SPACE mailbox  */
 
 int
-imap4d_select (struct imap4d_command *command, imap4d_tokbuf_t tok)
+imap4d_select (struct imap4d_session *session,
+               struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
   if (imap4d_tokbuf_argc (tok) != 3)
-    return util_finish (command, RESP_BAD, "Invalid arguments");
+    return io_completion_response (command, RESP_BAD, "Invalid arguments");
   return imap4d_select0 (command, imap4d_tokbuf_getarg (tok, IMAP4_ARG_1),
 			 MU_STREAM_RDWR);
 }
 
-/* This code is share with EXAMINE.  */
+/* This code is shared with EXAMINE.  */
 int
 imap4d_select0 (struct imap4d_command *command, const char *mboxname,
 		int flags)
@@ -48,8 +47,11 @@ imap4d_select0 (struct imap4d_command *command, const char *mboxname,
      currently selected mailbox without doing an expunge.  */
   if (mbox)
     {
+      imap4d_enter_critical ();
       mu_mailbox_sync (mbox);
       mu_mailbox_close (mbox);
+      manlock_unlock (mbox);
+      imap4d_leave_critical ();
       mu_mailbox_destroy (&mbox);
       /* Destroy the old uid table.  */
       imap4d_sync ();
@@ -57,13 +59,40 @@ imap4d_select0 (struct imap4d_command *command, const char *mboxname,
 
   if (strcmp (mboxname, "INBOX") == 0)
     flags |= MU_STREAM_CREAT;
-  mailbox_name = namespace_getfullpath (mboxname, "/", NULL);
+  mailbox_name = namespace_getfullpath (mboxname, NULL);
 
   if (!mailbox_name)
-    return util_finish (command, RESP_NO, "Couldn't open mailbox");
+    return io_completion_response (command, RESP_NO, "Couldn't open mailbox");
 
-  if ((status = mu_mailbox_create_default (&mbox, mailbox_name)) == 0
-      && (status = mu_mailbox_open (mbox, flags)) == 0)
+  if (flags & MU_STREAM_WRITE)
+    {
+      status = manlock_open_mailbox (&mbox, mailbox_name, 1, flags);
+      if (status)
+	flags &= ~MU_STREAM_WRITE;
+    }
+
+  if (!(flags & MU_STREAM_WRITE))
+    {
+      status = mu_mailbox_create_default (&mbox, mailbox_name);
+
+      if (status)
+	mu_diag_funcall (MU_DIAG_ERROR, "mu_mailbox_create_default",
+			 mailbox_name,
+			 status);
+      else
+	{
+	  status = mu_mailbox_open (mbox, flags);
+	  if (status)
+	    {
+	      mu_diag_funcall (MU_DIAG_ERROR, "mu_mailbox_open",
+			       mailbox_name,
+			       status);
+	      mu_mailbox_destroy (&mbox);
+	    }
+	}
+    }
+  
+  if (status == 0)
     {
       select_flags = flags;
       state = STATE_SEL;
@@ -74,14 +103,14 @@ imap4d_select0 (struct imap4d_command *command, const char *mboxname,
 	{
 	  free (mailbox_name);
 	  /* Need to set the state explicitely for select.  */
-	  return util_send ("%s OK [%s] %s Completed\r\n", command->tag,
-			    (flags & MU_STREAM_READ)  ?
-			    "READ-ONLY" : "READ-WRITE", command->name);
+	  return io_sendf ("%s OK [%s] %s Completed\n", command->tag,
+			   ((flags & MU_STREAM_RDWR) == MU_STREAM_RDWR) ?
+			   "READ-WRITE" : "READ-ONLY", command->name);
 	}
     }
   
   mu_mailbox_destroy (&mbox);
-  status = util_finish (command, RESP_NO, "Could not open %s: %s",
+  status = io_completion_response (command, RESP_NO, "Could not open %s: %s",
 			mboxname, mu_strerror (status));
   free (mailbox_name);
   return status;
@@ -92,7 +121,6 @@ int
 imap4d_select_status ()
 {
   const char *mflags = "\\Answered \\Flagged \\Deleted \\Seen \\Draft";
-  const char *pflags = "\\Answered \\Deleted \\Seen";
   unsigned long uidvalidity = 0;
   size_t count = 0, recent = 0, unseen = 0, uidnext = 0;
   int status = 0;
@@ -109,18 +137,22 @@ imap4d_select_status ()
 
   /* This outputs EXISTS and RECENT responses */
   imap4d_sync();
-  util_out (RESP_OK, "[UIDVALIDITY %lu] UID valididy status", uidvalidity);
-  util_out (RESP_OK, "[UIDNEXT %d] Predicted next uid", uidnext);
+  io_untagged_response (RESP_OK, "[UIDVALIDITY %lu] UID valididy status", 
+                           uidvalidity);
+  io_untagged_response (RESP_OK, "[UIDNEXT %lu] Predicted next uid",
+	                   (unsigned long) uidnext);
   if (unseen)
-    util_out (RESP_OK, "[UNSEEN %d] first unseen messsage ", unseen);
-  util_out (RESP_NONE, "FLAGS (%s)", mflags);
+    io_untagged_response (RESP_OK, "[UNSEEN %lu] first unseen message",
+	                     (unsigned long) unseen);
+  io_untagged_response (RESP_NONE, "FLAGS (%s)", mflags);
   /* FIXME:
      - '\*' can be supported if we use the attribute_set userflag()
      - Answered is still not set in the mailbox code.  */
-  if (select_flags & MU_STREAM_READ)
-    util_out (RESP_OK, "[PERMANENTFLAGS ()] No Permanent flags");
+  if (!(select_flags & MU_STREAM_WRITE))
+    io_untagged_response (RESP_OK, "[PERMANENTFLAGS ()] No permanent flags");
   else
-    util_out (RESP_OK, "[PERMANENTFLAGS (%s)] Permanent flags", pflags);
+    io_untagged_response (RESP_OK, "[PERMANENTFLAGS (%s)] Permanent flags",
+                          mflags);
 
   return 0;
 }
