@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004-2005, 2007, 2009-2012, 2014-2016 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,57 +13,39 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_SQL_H
 #define _MAILUTILS_SQL_H
 
 /* Configuration */
-enum mu_password_type
+enum mu_sql_password_encryption
   {
-    password_plaintext,       /* Plaintext passwords */
-    password_scrambled,       /* Scrambled MySQL (>=3.21) password */
-    password_hash,            /* MD5 (or DES or whatever) hash */
+    mu_sql_password_plaintext,       /* Plaintext passwords */
+    mu_sql_password_scrambled,       /* Scrambled MySQL (>=3.21) password */
+    mu_sql_password_hash,            /* MD5 (or DES or whatever) hash */
   };
 
 struct mu_sql_module_config
-{
-  char *interface;
-  char *getpwnam_query;
-  char *getpass_query;
-  char *getpwuid_query;
-  char *host; 
-  char *user;
-  char *passwd;
-  char *db;
-  int port;
-  enum mu_password_type password_type;
-  mu_assoc_t field_map;
-};
-
-/* FIXME: Should not be here, but needed for several other sources
-   (imap4d/auth_gsasl.c, for instance) */
-struct mu_internal_sql_config
 {
   int interface;
   char *getpwnam_query;
   char *getpass_query;
   char *getpwuid_query;
-  char *host; 
+  char *host;
   char *user;
   char *passwd;
   char *db;
   int port;
-  enum mu_password_type password_type;
+  enum mu_sql_password_encryption password_encryption;
   mu_assoc_t field_map;
 };
 
-extern struct mu_internal_sql_config mu_sql_module_config;
+extern struct mu_sql_module_config mu_sql_module_config;
 
 /* Loadable Modules Support */
-#define __s_cat2__(a,b) a ## b 
+#define __s_cat2__(a,b) a ## b
 #define __s_cat3__(a,b,c) a ## b ## c
 #define RDL_EXPORT(module,name) __s_cat3__(module,_LTX_,name)
 
@@ -105,7 +88,7 @@ struct mu_sql_dispatch
 {
   char *name;
   int port;
-  
+
   int (*init) (mu_sql_connection_t conn);
   int (*destroy) (mu_sql_connection_t conn);
 
@@ -115,7 +98,7 @@ struct mu_sql_dispatch
   int (*query) (mu_sql_connection_t conn, char *query);
   int (*store_result) (mu_sql_connection_t conn);
   int (*release_result) (mu_sql_connection_t conn);
-  
+
   int (*num_tuples) (mu_sql_connection_t conn, size_t *np);
   int (*num_columns) (mu_sql_connection_t conn, size_t *np);
 
@@ -124,12 +107,12 @@ struct mu_sql_dispatch
 
   int (*get_field_number) (mu_sql_connection_t conn, const char *fname,
 			   size_t *fno);
-  
+
   const char *(*errstr) (mu_sql_connection_t conn);
 };
 
 /* Public interfaces */
-int mu_sql_interface_index (char *name);
+int mu_sql_interface_index (char const *name);
 
 int mu_sql_connection_init (mu_sql_connection_t *conn, int interface,
 			    char *server, int  port, char *login,
@@ -149,7 +132,7 @@ int mu_sql_num_columns (mu_sql_connection_t conn, size_t *np);
 
 int mu_sql_get_column (mu_sql_connection_t conn, size_t nrow, size_t ncol,
 		       char **pdata);
-int mu_sql_get_field (mu_sql_connection_t conn, size_t nrow, char *fname,
+int mu_sql_get_field (mu_sql_connection_t conn, size_t nrow, const char *fname,
 		      char **pdata);
 
 const char *mu_sql_strerror (mu_sql_connection_t conn);
@@ -159,6 +142,4 @@ extern int mu_sql_getpass (const char *username, char **passwd);
 extern int mu_check_mysql_scrambled_password (const char *scrambled,
 					      const char *message);
 
-int mu_sql_decode_password_type (const char *arg, enum mu_password_type *t);
-
 #endif
