@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005, 2007, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2001-2003, 2005, 2007, 2010-2012, 2014-2016 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -38,7 +36,7 @@ mail_next (int argc, char **argv)
       n = get_cursor ();
       if (n == 0 || util_get_message (mbox, n, &msg))
 	{
-	  util_error (_("No applicable message"));
+	  mu_error (_("No applicable message"));
 	  return 1;
 	}
 
@@ -61,7 +59,7 @@ mail_next (int argc, char **argv)
 
       if (rc)
 	{
-	  util_error (_("No applicable message"));
+	  mu_error (_("No applicable message"));
 	  return 1;
 	}
     }
@@ -78,7 +76,7 @@ mail_next (int argc, char **argv)
 	}
       else
 	{
-	  util_error (_("No applicable message"));
+	  mu_error (_("No applicable message"));
 	  return 1;
 	}
     }
