@@ -490,7 +490,7 @@ mh_format (mh_format_t *fmt, mu_message_t msg, size_t msgno,
   
   mach.width = width - 1; /* Count the newline */
   mach.pc = 1;
-  mu_opool_create (&mach.pool, 1);
+  mu_opool_create (&mach.pool, MU_OPOOL_ENOMEMABRT);
   mu_list_create (&mach.addrlist);
   
   reset_fmt_defaults (&mach);
