@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
-   Software Foundation, Inc.
+   Copyright (C) 2002, 2005-2012, 2014-2016 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,28 +28,7 @@
 
 #include <dirent.h>
 
-static char doc[] = N_("GNU MH rmf")"\v"
-N_("Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = N_("[+FOLDER]");
-
-/* GNU options */
-static struct argp_option options[] = {
-  {"folder",  ARG_FOLDER, N_("FOLDER"), 0,
-   N_("specify the folder to delete")},
-  {"interactive", ARG_INTERACTIVE, N_("BOOL"), OPTION_ARG_OPTIONAL,
-    N_("interactive mode: ask for confirmation before removing each folder")},
-  {"nointeractive", ARG_NOINTERACTIVE, NULL, OPTION_HIDDEN, ""},
-  {"recursive", ARG_RECURSIVE, NULL, 0,
-   N_("recursively delete all subfolders")},
-  {"norecursive", ARG_NORECURSIVE, NULL, OPTION_HIDDEN, ""},
-  { 0 }
-};
-
-/* Traditional MH options */
-struct mh_option mh_option[] = {
-  { "interactive", MH_OPT_BOOL },
-  { 0 }
-};
+static char prog_doc[] = N_("Remove a GNU MH folder");
 
 int explicit_folder; /* Was the folder explicitly given */
 int interactive; /* Ask for confirmation before deleting */
@@ -59,40 +38,28 @@ static char *cur_folder_path; /* Full pathname of the current folder */
 static char *folder_name;     /* Name of the (topmost) folder to be
 				 deleted */
 
-static error_t
-opt_handler (int key, char *arg, struct argp_state *state)
+static void
+set_folder (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
-  switch (key)
-    {
-    case ARG_FOLDER:
-      explicit_folder = 1;
-      folder_name = arg;
-      break;
-
-    case ARG_INTERACTIVE:
-      interactive = is_true (arg);
-      break;
-
-    case ARG_NOINTERACTIVE:
-      interactive = 0;
-      break;
-	
-    case ARG_RECURSIVE:
-      recursive = is_true (arg);
-      break;
-      
-    case ARG_NORECURSIVE:
-      recursive = 0;
-      break;
-
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-  return 0;
-}
+  explicit_folder = 1;
+  folder_name = mu_strdup (arg);
+} 
 
+static struct mu_option options[] = {
+  { "folder",  0, N_("FOLDER"), MU_OPTION_DEFAULT,
+    N_("specify the folder to delete"),
+    mu_c_string, NULL, set_folder },
+  { "interactive", 0, NULL, MU_OPTION_DEFAULT,
+    N_("interactive mode: ask for confirmation before removing each folder"),
+    mu_c_bool, &interactive },
+  { "recursive", 0, NULL, MU_OPTION_DEFAULT,
+    N_("recursively delete all subfolders"),
+    mu_c_bool, &recursive },
+  MU_OPTION_END
+};
+
 static char *
-current_folder_path ()
+current_folder_path (void)
 {
   mu_mailbox_t mbox = mh_open_folder (mh_current_folder (), MU_STREAM_RDWR);
   mu_url_t url;
@@ -182,12 +149,7 @@ main (int argc, char **argv)
   int status;
   char *name;
 
-  /* Native Language Support */
-  MU_APP_INIT_NLS ();
-
-  mh_argp_init ();
-  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
-		 opt_handler, NULL, NULL);
+  mh_getopt (&argc, &argv, options, 0, NULL, prog_doc, NULL);
 
   cur_folder_path = current_folder_path ();
 
@@ -197,7 +159,7 @@ main (int argc, char **argv)
       name = cur_folder_path;
     }
   else
-    name = mh_expand_name (NULL, folder_name, 0);
+    name = mh_expand_name (NULL, folder_name, NAME_FOLDER);
   if (recursive)
     status = recrmf (name);
   else
