@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012 Free Software Foundation, Inc.
+   Copyright (C) 2010-2012, 2014-2016 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,38 +29,8 @@
 #include <mailutils/imaputil.h>
 #include <mailutils/msgset.h>
 #include "mu.h"
-#include "argp.h"
-#include "xalloc.h"
 
-static char imap_doc[] = N_("mu imap - IMAP4 client shell.");
 char imap_docstring[] = N_("IMAP4 client shell");
-static char imap_args_doc[] = "";
-
-static struct argp_option imap_options[] = {
-  { NULL }
-};
-
-static error_t
-imap_parse_opt (int key, char *arg, struct argp_state *state)
-{
-  switch (key)
-    {
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-  return 0;
-}
-
-static struct argp imap_argp = {
-  imap_options,
-  imap_parse_opt,
-  imap_args_doc,
-  imap_doc,
-  NULL,
-  NULL,
-  NULL
-};
-
 
 static mu_imap_t imap;
 static int uid_mode;
@@ -155,7 +125,7 @@ imap_prompt_env ()
 {
   enum mu_imap_session_state state = current_imap_state ();
   if (!mutool_prompt_env)
-    mutool_prompt_env = xcalloc (2*7 + 1, sizeof(mutool_prompt_env[0]));
+    mutool_prompt_env = mu_calloc (2*7 + 1, sizeof(mutool_prompt_env[0]));
 
   mutool_prompt_env[0] = "user";
   mutool_prompt_env[1] = (state >= MU_IMAP_SESSION_AUTH && username) ?
@@ -618,9 +588,9 @@ com_connect (int argc, char **argv)
   if (!status)
     {
       connect_argc = argc;
-      connect_argv = xcalloc (argc, sizeof (*connect_argv));
+      connect_argv = mu_calloc (argc, sizeof (*connect_argv));
       for (i = 0; i < argc; i++)
-	connect_argv[i] = xstrdup (argv[i]);
+	connect_argv[i] = mu_strdup (argv[i]);
       connect_argv[i] = NULL;
     
       imap_prompt_env ();
@@ -1121,6 +1091,35 @@ com_copy (int argc, char **argv)
     }
   return 0;
 }
+
+static int
+com_search (int argc, char **argv)
+{
+  mu_msgset_t mset;
+  size_t count;
+  int rc;
+  
+  rc = mu_imap_search (imap, uid_mode, argv[1], &mset);
+  if (rc)
+    {
+      report_failure ("search", rc);
+      return 0;
+    }
+	
+  rc = mu_msgset_count (mset, &count);
+  if (rc == EINVAL || count == 0)
+    {
+      mu_printf ("%s\n", _("no matches"));
+      return 0;
+    }
+  mu_printf ("%lu matches:", (unsigned long) count);
+  mu_msgset_print (mu_strout, mset);
+  mu_printf ("\n");
+  mu_msgset_free (mset);
+  
+  return 0;
+}
+
 
 static int
 print_list_item (void *item, void *data)
@@ -1294,6 +1293,10 @@ struct mutool_command imap_comtab[] = {
     com_uid,
     N_("[on|off]"),
     N_("control UID mode") },
+  { "search",       2, 2, CMD_COALESCE_EXTRA_ARGS,
+    com_search,
+    N_("args..."),
+    N_("search the mailbox") },
   { "quit",         1, 1, 0,
     com_logout,
     NULL,
@@ -1304,13 +1307,7 @@ struct mutool_command imap_comtab[] = {
 int
 mutool_imap (int argc, char **argv)
 {
-  int index;
-
-  if (argp_parse (&imap_argp, argc, argv, ARGP_IN_ORDER, &index, NULL))
-    return 1;
-
-  argc -= index;
-  argv += index;
+  mu_action_getopt (&argc, &argv, NULL, imap_docstring, NULL);
 
   if (argc)
     {
@@ -1319,7 +1316,7 @@ mutool_imap (int argc, char **argv)
     }
 
   /* Command line prompt */
-  mutool_shell_prompt = xstrdup ("imap> ");
+  mutool_shell_prompt = mu_strdup ("imap> ");
   imap_prompt_env ();
   mutool_shell ("imap", imap_comtab);
   return 0;
