@@ -1,6 +1,5 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2009-2012, 2014-2016 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,10 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #include "libmu_py.h"
 
@@ -134,9 +131,4 @@ _mu_py_attach_registrar ()
 
   mu_registrar_record (MU_DEFAULT_RECORD);
   mu_registrar_set_default_record (MU_DEFAULT_RECORD);
-
-#ifdef WITH_TLS
-  mu_init_tls_libs ();
-#endif /* WITH_TLS */
-
 }
